pub struct EventLoop<T: 'static> { /* private fields */ }Expand description
Provides a way to retrieve events from the system and from the windows that were registered to the events loop.
An EventLoop can be seen more or less as a “context”. Calling EventLoop::new
initializes everything that will be required to create windows. For example on Linux creating
an event loop opens a connection to the X or Wayland server.
To wake up an EventLoop from a another thread, see the EventLoopProxy docs.
Note that this cannot be shared across threads (due to platform-dependant logic
forbidding it), as such it is neither Send nor Sync. If you need cross-thread access,
the Window created from this can be sent to an other thread, and the
EventLoopProxy allows you to wake up an EventLoop from another thread.
Implementations
Sourceimpl EventLoop<()>
impl EventLoop<()>
Sourcepub fn new() -> Result<EventLoop<()>, EventLoopError>
pub fn new() -> Result<EventLoop<()>, EventLoopError>
Create the event loop.
This is an alias of EventLoop::builder().build().
Examples found in repository?
33fn main() -> Result<(), impl std::error::Error> {
34 #[cfg(web_platform)]
35 console_error_panic_hook::set_once();
36
37 tracing::init();
38
39 info!("Press '1' to switch to Wait mode.");
40 info!("Press '2' to switch to WaitUntil mode.");
41 info!("Press '3' to switch to Poll mode.");
42 info!("Press 'R' to toggle request_redraw() calls.");
43 info!("Press 'Esc' to close the window.");
44
45 let event_loop = EventLoop::new().unwrap();
46
47 let mut app = ControlFlowDemo::default();
48 event_loop.run_app(&mut app)
49}More examples
5fn main() -> std::process::ExitCode {
6 use std::process::ExitCode;
7 use std::thread::sleep;
8 use std::time::Duration;
9
10 use rio_winit_fork::application::ApplicationHandler;
11 use rio_winit_fork::event::WindowEvent;
12 use rio_winit_fork::event_loop::{ActiveEventLoop, EventLoop};
13 use rio_winit_fork::platform::pump_events::{EventLoopExtPumpEvents, PumpStatus};
14 use rio_winit_fork::window::{Window, WindowId};
15
16 #[path = "util/fill.rs"]
17 mod fill;
18
19 #[derive(Default)]
20 struct PumpDemo {
21 window: Option<Window>,
22 }
23
24 impl ApplicationHandler for PumpDemo {
25 fn resumed(&mut self, event_loop: &ActiveEventLoop) {
26 let window_attributes = Window::default_attributes().with_title("A fantastic window!");
27 self.window = Some(event_loop.create_window(window_attributes).unwrap());
28 }
29
30 fn window_event(
31 &mut self,
32 event_loop: &ActiveEventLoop,
33 _window_id: WindowId,
34 event: WindowEvent,
35 ) {
36 println!("{event:?}");
37
38 let window = match self.window.as_ref() {
39 Some(window) => window,
40 None => return,
41 };
42
43 match event {
44 WindowEvent::CloseRequested => event_loop.exit(),
45 WindowEvent::RedrawRequested => {
46 fill::fill_window(window);
47 window.request_redraw();
48 },
49 _ => (),
50 }
51 }
52 }
53
54 let mut event_loop = EventLoop::new().unwrap();
55
56 tracing_subscriber::fmt::init();
57
58 let mut app = PumpDemo::default();
59
60 loop {
61 let timeout = Some(Duration::ZERO);
62 let status = event_loop.pump_app_events(timeout, &mut app);
63
64 if let PumpStatus::Exit(exit_code) = status {
65 break ExitCode::from(exit_code as u8);
66 }
67
68 // Sleep for 1/60 second to simulate application work
69 //
70 // Since `pump_events` doesn't block it will be important to
71 // throttle the loop in the app somehow.
72 println!("Update()");
73 sleep(Duration::from_millis(16));
74 }
75}5fn main() -> Result<(), Box<dyn std::error::Error>> {
6 use std::time::Duration;
7
8 use rio_winit_fork::application::ApplicationHandler;
9 use rio_winit_fork::event::WindowEvent;
10 use rio_winit_fork::event_loop::{ActiveEventLoop, EventLoop};
11 use rio_winit_fork::platform::run_on_demand::EventLoopExtRunOnDemand;
12 use rio_winit_fork::window::{Window, WindowId};
13
14 #[path = "util/fill.rs"]
15 mod fill;
16
17 #[derive(Default)]
18 struct App {
19 idx: usize,
20 window_id: Option<WindowId>,
21 window: Option<Window>,
22 }
23
24 impl ApplicationHandler for App {
25 fn about_to_wait(&mut self, _event_loop: &ActiveEventLoop) {
26 if let Some(window) = self.window.as_ref() {
27 window.request_redraw();
28 }
29 }
30
31 fn resumed(&mut self, event_loop: &ActiveEventLoop) {
32 let window_attributes = Window::default_attributes()
33 .with_title("Fantastic window number one!")
34 .with_inner_size(rio_winit_fork::dpi::LogicalSize::new(128.0, 128.0));
35 let window = event_loop.create_window(window_attributes).unwrap();
36 self.window_id = Some(window.id());
37 self.window = Some(window);
38 }
39
40 fn window_event(
41 &mut self,
42 event_loop: &ActiveEventLoop,
43 window_id: WindowId,
44 event: WindowEvent,
45 ) {
46 if event == WindowEvent::Destroyed && self.window_id == Some(window_id) {
47 println!(
48 "--------------------------------------------------------- Window {} Destroyed",
49 self.idx
50 );
51 self.window_id = None;
52 event_loop.exit();
53 return;
54 }
55
56 let window = match self.window.as_mut() {
57 Some(window) => window,
58 None => return,
59 };
60
61 match event {
62 WindowEvent::CloseRequested => {
63 println!(
64 "--------------------------------------------------------- Window {} \
65 CloseRequested",
66 self.idx
67 );
68 fill::cleanup_window(window);
69 self.window = None;
70 },
71 WindowEvent::RedrawRequested => {
72 fill::fill_window(window);
73 },
74 _ => (),
75 }
76 }
77 }
78
79 tracing_subscriber::fmt::init();
80
81 let mut event_loop = EventLoop::new().unwrap();
82
83 let mut app = App { idx: 1, ..Default::default() };
84 event_loop.run_app_on_demand(&mut app)?;
85
86 println!("--------------------------------------------------------- Finished first loop");
87 println!("--------------------------------------------------------- Waiting 5 seconds");
88 std::thread::sleep(Duration::from_secs(5));
89
90 app.idx += 1;
91 event_loop.run_app_on_demand(&mut app)?;
92 println!("--------------------------------------------------------- Finished second loop");
93 Ok(())
94}3fn main() -> Result<(), impl std::error::Error> {
4 use std::collections::HashMap;
5
6 use rio_winit_fork::dpi::{LogicalPosition, LogicalSize, Position};
7 use rio_winit_fork::event::{ElementState, Event, KeyEvent, WindowEvent};
8 use rio_winit_fork::event_loop::{ActiveEventLoop, EventLoop};
9 use rio_winit_fork::raw_window_handle::HasRawWindowHandle;
10 use rio_winit_fork::window::Window;
11
12 #[path = "util/fill.rs"]
13 mod fill;
14
15 fn spawn_child_window(parent: &Window, event_loop: &ActiveEventLoop) -> Window {
16 let parent = parent.raw_window_handle().unwrap();
17 let mut window_attributes = Window::default_attributes()
18 .with_title("child window")
19 .with_inner_size(LogicalSize::new(200.0f32, 200.0f32))
20 .with_position(Position::Logical(LogicalPosition::new(0.0, 0.0)))
21 .with_visible(true);
22 // `with_parent_window` is unsafe. Parent window must be a valid window.
23 window_attributes = unsafe { window_attributes.with_parent_window(Some(parent)) };
24
25 event_loop.create_window(window_attributes).unwrap()
26 }
27
28 let mut windows = HashMap::new();
29
30 let event_loop: EventLoop<()> = EventLoop::new().unwrap();
31 let mut parent_window_id = None;
32
33 event_loop.run(move |event: Event<()>, event_loop| {
34 match event {
35 Event::Resumed => {
36 let attributes = Window::default_attributes()
37 .with_title("parent window")
38 .with_position(Position::Logical(LogicalPosition::new(0.0, 0.0)))
39 .with_inner_size(LogicalSize::new(640.0f32, 480.0f32));
40 let window = event_loop.create_window(attributes).unwrap();
41
42 parent_window_id = Some(window.id());
43
44 println!("Parent window id: {parent_window_id:?})");
45 windows.insert(window.id(), window);
46 },
47 Event::WindowEvent { window_id, event } => match event {
48 WindowEvent::CloseRequested => {
49 windows.clear();
50 event_loop.exit();
51 },
52 WindowEvent::CursorEntered { device_id: _ } => {
53 // On x11, println when the cursor entered in a window even if the child window
54 // is created by some key inputs.
55 // the child windows are always placed at (0, 0) with size (200, 200) in the
56 // parent window, so we also can see this log when we move
57 // the cursor around (200, 200) in parent window.
58 println!("cursor entered in the window {window_id:?}");
59 },
60 WindowEvent::KeyboardInput {
61 event: KeyEvent { state: ElementState::Pressed, .. },
62 ..
63 } => {
64 let parent_window = windows.get(&parent_window_id.unwrap()).unwrap();
65 let child_window = spawn_child_window(parent_window, event_loop);
66 let child_id = child_window.id();
67 println!("Child window created with id: {child_id:?}");
68 windows.insert(child_id, child_window);
69 },
70 WindowEvent::RedrawRequested => {
71 if let Some(window) = windows.get(&window_id) {
72 fill::fill_window(window);
73 }
74 },
75 _ => (),
76 },
77 _ => (),
78 }
79 })
80}Sourcepub fn builder() -> EventLoopBuilder<()>
pub fn builder() -> EventLoopBuilder<()>
Start building a new event loop.
This returns an EventLoopBuilder, to allow configuring the event loop before creation.
To get the actual event loop, call build on that.
Sourceimpl<T> EventLoop<T>
impl<T> EventLoop<T>
Sourcepub fn with_user_event() -> EventLoopBuilder<T>
pub fn with_user_event() -> EventLoopBuilder<T>
Start building a new event loop, with the given type as the user event type.
Examples found in repository?
41fn main() -> Result<(), Box<dyn Error>> {
42 #[cfg(web_platform)]
43 console_error_panic_hook::set_once();
44
45 tracing::init();
46
47 let event_loop = EventLoop::<UserEvent>::with_user_event().build()?;
48 let _event_loop_proxy = event_loop.create_proxy();
49
50 // Wire the user event from another thread.
51 #[cfg(not(web_platform))]
52 std::thread::spawn(move || {
53 // Wake up the `event_loop` once every second and dispatch a custom event
54 // from a different thread.
55 info!("Starting to send user event every second");
56 loop {
57 let _ = _event_loop_proxy.send_event(UserEvent::WakeUp);
58 std::thread::sleep(std::time::Duration::from_secs(1));
59 }
60 });
61
62 let mut state = Application::new(&event_loop);
63
64 event_loop.run_app(&mut state).map_err(Into::into)
65}Sourcepub fn run<F>(self, event_handler: F) -> Result<(), EventLoopError>
👎Deprecated: use EventLoop::run_app insteadAvailable on not (web_platform and target feature exception-handling).
pub fn run<F>(self, event_handler: F) -> Result<(), EventLoopError>
EventLoop::run_app insteadweb_platform and target feature exception-handling).See run_app.
Examples found in repository?
3fn main() -> Result<(), impl std::error::Error> {
4 use std::collections::HashMap;
5
6 use rio_winit_fork::dpi::{LogicalPosition, LogicalSize, Position};
7 use rio_winit_fork::event::{ElementState, Event, KeyEvent, WindowEvent};
8 use rio_winit_fork::event_loop::{ActiveEventLoop, EventLoop};
9 use rio_winit_fork::raw_window_handle::HasRawWindowHandle;
10 use rio_winit_fork::window::Window;
11
12 #[path = "util/fill.rs"]
13 mod fill;
14
15 fn spawn_child_window(parent: &Window, event_loop: &ActiveEventLoop) -> Window {
16 let parent = parent.raw_window_handle().unwrap();
17 let mut window_attributes = Window::default_attributes()
18 .with_title("child window")
19 .with_inner_size(LogicalSize::new(200.0f32, 200.0f32))
20 .with_position(Position::Logical(LogicalPosition::new(0.0, 0.0)))
21 .with_visible(true);
22 // `with_parent_window` is unsafe. Parent window must be a valid window.
23 window_attributes = unsafe { window_attributes.with_parent_window(Some(parent)) };
24
25 event_loop.create_window(window_attributes).unwrap()
26 }
27
28 let mut windows = HashMap::new();
29
30 let event_loop: EventLoop<()> = EventLoop::new().unwrap();
31 let mut parent_window_id = None;
32
33 event_loop.run(move |event: Event<()>, event_loop| {
34 match event {
35 Event::Resumed => {
36 let attributes = Window::default_attributes()
37 .with_title("parent window")
38 .with_position(Position::Logical(LogicalPosition::new(0.0, 0.0)))
39 .with_inner_size(LogicalSize::new(640.0f32, 480.0f32));
40 let window = event_loop.create_window(attributes).unwrap();
41
42 parent_window_id = Some(window.id());
43
44 println!("Parent window id: {parent_window_id:?})");
45 windows.insert(window.id(), window);
46 },
47 Event::WindowEvent { window_id, event } => match event {
48 WindowEvent::CloseRequested => {
49 windows.clear();
50 event_loop.exit();
51 },
52 WindowEvent::CursorEntered { device_id: _ } => {
53 // On x11, println when the cursor entered in a window even if the child window
54 // is created by some key inputs.
55 // the child windows are always placed at (0, 0) with size (200, 200) in the
56 // parent window, so we also can see this log when we move
57 // the cursor around (200, 200) in parent window.
58 println!("cursor entered in the window {window_id:?}");
59 },
60 WindowEvent::KeyboardInput {
61 event: KeyEvent { state: ElementState::Pressed, .. },
62 ..
63 } => {
64 let parent_window = windows.get(&parent_window_id.unwrap()).unwrap();
65 let child_window = spawn_child_window(parent_window, event_loop);
66 let child_id = child_window.id();
67 println!("Child window created with id: {child_id:?}");
68 windows.insert(child_id, child_window);
69 },
70 WindowEvent::RedrawRequested => {
71 if let Some(window) = windows.get(&window_id) {
72 fill::fill_window(window);
73 }
74 },
75 _ => (),
76 },
77 _ => (),
78 }
79 })
80}Sourcepub fn run_app<A: ApplicationHandler<T>>(
self,
app: &mut A,
) -> Result<(), EventLoopError>
Available on not (web_platform and target feature exception-handling).
pub fn run_app<A: ApplicationHandler<T>>( self, app: &mut A, ) -> Result<(), EventLoopError>
web_platform and target feature exception-handling).Run the application with the event loop on the calling thread.
See the set_control_flow() docs on how to change the event loop’s behavior.
Platform-specific
-
iOS: Will never return to the caller and so values not passed to this function will not be dropped before the process exits.
-
Web: Will act as if it never returns to the caller by throwing a Javascript exception (that Rust doesn’t see) that will also mean that the rest of the function is never executed and any values not passed to this function will not be dropped.
Web applications are recommended to use
EventLoopExtWebSys::spawn()1 instead ofrun_app()to avoid the need for the Javascript exception trick, and to make it clearer that the event loop runs asynchronously (via the browser’s own, internal, event loop) and doesn’t block the current thread of execution like it does on other platforms.This function won’t be available with
target_feature = "exception-handling".
EventLoopExtWebSys::spawn_app()is only available on Web. ↩
Examples found in repository?
33fn main() -> Result<(), impl std::error::Error> {
34 #[cfg(web_platform)]
35 console_error_panic_hook::set_once();
36
37 tracing::init();
38
39 info!("Press '1' to switch to Wait mode.");
40 info!("Press '2' to switch to WaitUntil mode.");
41 info!("Press '3' to switch to Poll mode.");
42 info!("Press 'R' to toggle request_redraw() calls.");
43 info!("Press 'Esc' to close the window.");
44
45 let event_loop = EventLoop::new().unwrap();
46
47 let mut app = ControlFlowDemo::default();
48 event_loop.run_app(&mut app)
49}More examples
41fn main() -> Result<(), Box<dyn Error>> {
42 #[cfg(web_platform)]
43 console_error_panic_hook::set_once();
44
45 tracing::init();
46
47 let event_loop = EventLoop::<UserEvent>::with_user_event().build()?;
48 let _event_loop_proxy = event_loop.create_proxy();
49
50 // Wire the user event from another thread.
51 #[cfg(not(web_platform))]
52 std::thread::spawn(move || {
53 // Wake up the `event_loop` once every second and dispatch a custom event
54 // from a different thread.
55 info!("Starting to send user event every second");
56 loop {
57 let _ = _event_loop_proxy.send_event(UserEvent::WakeUp);
58 std::thread::sleep(std::time::Duration::from_secs(1));
59 }
60 });
61
62 let mut state = Application::new(&event_loop);
63
64 event_loop.run_app(&mut state).map_err(Into::into)
65}Sourcepub fn create_proxy(&self) -> EventLoopProxy<T>
pub fn create_proxy(&self) -> EventLoopProxy<T>
Creates an EventLoopProxy that can be used to dispatch user events
to the main event loop, possibly from another thread.
Examples found in repository?
41fn main() -> Result<(), Box<dyn Error>> {
42 #[cfg(web_platform)]
43 console_error_panic_hook::set_once();
44
45 tracing::init();
46
47 let event_loop = EventLoop::<UserEvent>::with_user_event().build()?;
48 let _event_loop_proxy = event_loop.create_proxy();
49
50 // Wire the user event from another thread.
51 #[cfg(not(web_platform))]
52 std::thread::spawn(move || {
53 // Wake up the `event_loop` once every second and dispatch a custom event
54 // from a different thread.
55 info!("Starting to send user event every second");
56 loop {
57 let _ = _event_loop_proxy.send_event(UserEvent::WakeUp);
58 std::thread::sleep(std::time::Duration::from_secs(1));
59 }
60 });
61
62 let mut state = Application::new(&event_loop);
63
64 event_loop.run_app(&mut state).map_err(Into::into)
65}Sourcepub fn owned_display_handle(&self) -> OwnedDisplayHandle
pub fn owned_display_handle(&self) -> OwnedDisplayHandle
Gets a persistent reference to the underlying platform display.
See the OwnedDisplayHandle type for more information.
Sourcepub fn listen_device_events(&self, allowed: DeviceEvents)
pub fn listen_device_events(&self, allowed: DeviceEvents)
Change if or when DeviceEvents are captured.
See ActiveEventLoop::listen_device_events for details.
Sourcepub fn set_control_flow(&self, control_flow: ControlFlow)
pub fn set_control_flow(&self, control_flow: ControlFlow)
Sets the ControlFlow.
Sourcepub fn create_window(
&self,
window_attributes: WindowAttributes,
) -> Result<Window, OsError>
👎Deprecated: use ActiveEventLoop::create_window instead
pub fn create_window( &self, window_attributes: WindowAttributes, ) -> Result<Window, OsError>
ActiveEventLoop::create_window insteadCreate a window.
Creating window without event loop running often leads to improper window creation;
use ActiveEventLoop::create_window instead.
Sourcepub fn create_custom_cursor(
&self,
custom_cursor: CustomCursorSource,
) -> CustomCursor
pub fn create_custom_cursor( &self, custom_cursor: CustomCursorSource, ) -> CustomCursor
Create custom cursor.
Examples found in repository?
88 fn new<T>(event_loop: &EventLoop<T>) -> Self {
89 // SAFETY: we drop the context right before the event loop is stopped, thus making it safe.
90 #[cfg(not(any(android_platform, ios_platform)))]
91 let context = Some(
92 Context::new(unsafe {
93 std::mem::transmute::<DisplayHandle<'_>, DisplayHandle<'static>>(
94 event_loop.display_handle().unwrap(),
95 )
96 })
97 .unwrap(),
98 );
99
100 // You'll have to choose an icon size at your own discretion. On X11, the desired size
101 // varies by WM, and on Windows, you still have to account for screen scaling. Here
102 // we use 32px, since it seems to work well enough in most cases. Be careful about
103 // going too high, or you'll be bitten by the low-quality downscaling built into the
104 // WM.
105 let icon = load_icon(include_bytes!("data/icon.png"));
106
107 info!("Loading cursor assets");
108 let custom_cursors = vec![
109 event_loop.create_custom_cursor(decode_cursor(include_bytes!("data/cross.png"))),
110 event_loop.create_custom_cursor(decode_cursor(include_bytes!("data/cross2.png"))),
111 event_loop.create_custom_cursor(decode_cursor(include_bytes!("data/gradient.png"))),
112 ];
113
114 Self {
115 #[cfg(not(any(android_platform, ios_platform)))]
116 context,
117 custom_cursors,
118 icon,
119 windows: Default::default(),
120 }
121 }Trait Implementations
Sourceimpl<T: 'static> EventLoopExtIOS for EventLoop<T>
Available on ios_platform only.
impl<T: 'static> EventLoopExtIOS for EventLoop<T>
ios_platform only.Sourceimpl<T> EventLoopExtPumpEvents for EventLoop<T>
Available on windows_platform or macos_platform or android_platform or x11_platform or wayland_platform only.
impl<T> EventLoopExtPumpEvents for EventLoop<T>
windows_platform or macos_platform or android_platform or x11_platform or wayland_platform only.Sourcetype UserEvent = T
type UserEvent = T
Event::UserEvent.Sourcefn pump_events<F>(
&mut self,
timeout: Option<Duration>,
event_handler: F,
) -> PumpStatus
fn pump_events<F>( &mut self, timeout: Option<Duration>, event_handler: F, ) -> PumpStatus
pump_app_events.Sourcefn pump_app_events<A: ApplicationHandler<Self::UserEvent>>(
&mut self,
timeout: Option<Duration>,
app: &mut A,
) -> PumpStatus
fn pump_app_events<A: ApplicationHandler<Self::UserEvent>>( &mut self, timeout: Option<Duration>, app: &mut A, ) -> PumpStatus
EventLoop to check for and dispatch pending events. Read moreSourceimpl<T> EventLoopExtRunOnDemand for EventLoop<T>
Available on windows_platform or macos_platform or android_platform or x11_platform or wayland_platform only.
impl<T> EventLoopExtRunOnDemand for EventLoop<T>
windows_platform or macos_platform or android_platform or x11_platform or wayland_platform only.Sourcetype UserEvent = T
type UserEvent = T
Event::UserEvent.Sourcefn run_on_demand<F>(&mut self, event_handler: F) -> Result<(), EventLoopError>
fn run_on_demand<F>(&mut self, event_handler: F) -> Result<(), EventLoopError>
run_app_on_demand.Sourcefn run_app_on_demand<A: ApplicationHandler<Self::UserEvent>>(
&mut self,
app: &mut A,
) -> Result<(), EventLoopError>
fn run_app_on_demand<A: ApplicationHandler<Self::UserEvent>>( &mut self, app: &mut A, ) -> Result<(), EventLoopError>
Sourceimpl<T> EventLoopExtWebSys for EventLoop<T>
Available on web_platform only.
impl<T> EventLoopExtWebSys for EventLoop<T>
web_platform only.Sourceimpl<T> HasDisplayHandle for EventLoop<T>
Available on crate feature rwh_06 only.
impl<T> HasDisplayHandle for EventLoop<T>
rwh_06 only.Sourcefn display_handle(&self) -> Result<DisplayHandle<'_>, HandleError>
fn display_handle(&self) -> Result<DisplayHandle<'_>, HandleError>
impl<T> EventLoopExtAndroid for EventLoop<T>
android_platform only.Auto Trait Implementations
impl<T> Freeze for EventLoop<T>
impl<T> !RefUnwindSafe for EventLoop<T>
impl<T> !Send for EventLoop<T>
impl<T> !Sync for EventLoop<T>
impl<T> Unpin for EventLoop<T>
impl<T> !UnwindSafe for EventLoop<T>